
/* Joyduino host by Marq/Fit

15.10.2011: Accept port as a parameter, minor cleanup of serial code
6.10.2011: Initial revision
*/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>

#include <SDL.h>

#define BPS B115200     // Serial speed

#define JOY1 128
#define JOY2 64

// Bits for joy direction/button bytes
#define JOY_UP    1
#define JOY_DOWN  2
#define JOY_LEFT  4
#define JOY_RIGHT 8
#define JOY_B1    16
#define JOY_B2    32

// Default Arduino Uno port for Linux
#ifdef __linux__
#define PORT "/dev/ttyACM0"
#endif

// Mac OS X
#ifdef __APPLE__
#define PORT "/dev/cu.usbmodem621"
#endif

// The Microsoft attempt at an OS (untested)
#ifdef __WIN32__
#define PORT "COM3"
#endif

void put(int f,unsigned char c)
{
    write(f,&c,1);
    tcdrain(f);
}

int main(int argc,char *argv[])
{
    int f,quit=0;
    unsigned char c,
                  joy1=JOY1,joy2=JOY2;
    struct termios opt;
    char *portname=PORT;

    SDL_Event e;

    printf("Joyduino 0.01\n");

    if(argc>1)
        portname=argv[1];

    f=open(portname,O_RDWR|O_NOCTTY); // Open Arduino
    if(f==-1)
    {
        printf("Arduino can't be opened at %s\n",portname);
        return(1);
    }
    fcntl(f,F_SETFL,0);
    printf("Arduino opened\n");

    tcgetattr(f,&opt); // Set serial params
    cfsetispeed(&opt,BPS);
    cfsetospeed(&opt,BPS);
    opt.c_cflag |= (CLOCAL | CREAD | CS8);
    opt.c_iflag |= ICRNL;

    if(tcsetattr(f, TCSANOW, &opt)<0)
    {
        printf("Serial attribs can't be set\n");
        return(1);
    }

    printf("Waiting for the board to reset\n");
    c=' ';
    do
    {
        read(f,&c,1);
    }
    while(c!='!'); // The greeting message ends in '!'

    printf("Start SDL\n");
    SDL_Init(SDL_INIT_VIDEO);
    SDL_SetVideoMode(100,100,32,0);
    SDL_WM_SetCaption("Joyduino",0);

    while(!quit)
    {
        while(SDL_PollEvent(&e)>0)
        {
            if(e.type==SDL_KEYDOWN)
            {
                switch(e.key.keysym.sym)
                {
                    case SDLK_ESCAPE: quit=1; break;

                    // Joy 1
                    case SDLK_UP:    joy1|=JOY_UP; break;
                    case SDLK_DOWN:  joy1|=JOY_DOWN; break;
                    case SDLK_LEFT:  joy1|=JOY_LEFT; break;
                    case SDLK_RIGHT: joy1|=JOY_RIGHT; break;
                    case SDLK_SPACE: joy1|=JOY_B1; break;

                    // Joy 2
                    case SDLK_w:      joy2|=JOY_UP; break;
                    case SDLK_s:      joy2|=JOY_DOWN; break;
                    case SDLK_a:      joy2|=JOY_LEFT; break;
                    case SDLK_d:      joy2|=JOY_RIGHT; break;
                    case SDLK_LSHIFT: joy2|=JOY_B1; break;

                    default: break;
                }
            }
            if(e.type==SDL_KEYUP)
            {
                switch(e.key.keysym.sym)
                {
                    // Joy 1
                    case SDLK_UP:    joy1&=(~JOY_UP); break;
                    case SDLK_DOWN:  joy1&=(~JOY_DOWN); break;
                    case SDLK_LEFT:  joy1&=(~JOY_LEFT); break;
                    case SDLK_RIGHT: joy1&=(~JOY_RIGHT); break;
                    case SDLK_SPACE: joy1&=(~JOY_B1); break;

                    // Joy 2
                    case SDLK_w:      joy2&=(~JOY_UP); break;
                    case SDLK_s:      joy2&=(~JOY_DOWN); break;
                    case SDLK_a:      joy2&=(~JOY_LEFT); break;
                    case SDLK_d:      joy2&=(~JOY_RIGHT); break;
                    case SDLK_LSHIFT: joy2&=(~JOY_B1); break;

                    default: break;
                }
            }
        }

        put(f,joy1);
        put(f,joy2);
    }

    SDL_Quit();
    close(f);
    return(0);
}
